from markupsafe import Markup

from brigid.jinja2_render.utils import jinjaglobal
from brigid.library.storage import storage
from brigid.plugins.i18n.default_translations import translations


@jinjaglobal
def translate_tag(language: str, tag: str) -> str:
    site = storage.get_site()
    return site.languages[language].tags_translations[tag]


def _translate(language: str, text_id: str) -> str:
    site = storage.get_site()

    if text_id in site.languages[language].theme_translations:
        return site.languages[language].theme_translations[text_id]

    if language not in translations:
        language = site.default_language

    if language not in translations:
        language = "en"

    if text_id in translations[language]:
        return translations[language][text_id]

    return text_id


@jinjaglobal
def translate_theme(language: str, text_id: str) -> str:
    return Markup(_translate(language, text_id))



import pytest
from bs4 import BeautifulSoup
from html5lib import HTMLParser

from brigid.domain import request_context
from brigid.domain.urls import UrlsPost, UrlsTags
from brigid.jinja2_render.entities import IndexInfo, Info, MetaInfo, PageInfo, Template
from brigid.jinja2_render.templates import render
from brigid.library.tests import make as library_make


def assert_correct_html(text: str) -> None:
    parser = HTMLParser(strict=True)

    try:
        parser.parse(text)
    except Exception as e:
        pytest.fail(f"Malformed HTML: {e}")


def test_assert_correct_html() -> None:
    from _pytest.outcomes import Failed

    assert_correct_html("<!DOCTYPE html><html></html>")

    with pytest.raises(Failed):
        assert_correct_html("<html>")


class TestPageRender:

    def test_page_rendered_without_errors(self) -> None:
        article = library_make.article()
        page = library_make.page(article, body="bla-bla-text")

        # TODO: make real meta_info
        #       maybe move code of constructing meta_info (from renderers.py) to separate function
        meta_info = MetaInfo(
            site_title="title",  # TODO: get real site title
            language=page.language,
            allowed_languages=[page.language],
            title=page.title,
            seo_description=page.seo_description,
            author="author",
            # TODO: add some tags
            tags=[],
            published_at=page.published_at,
            seo_image_url="",
        )

        post_url = UrlsPost(language=page.language, slug=article.slug)

        info = Info(language=page.language, current_url=post_url)

        page_info = PageInfo(
            article=article,
            page=page,
            similar_pages=[],
        )

        request_context.set("language", page.language)
        request_context.set("url", post_url)

        content = render(
            str(Template.article_page),
            {
                "info": info,
                "meta_info": meta_info,
                "page_info": page_info,
                "index_info": None,
            },
        )

        assert content is not None

        assert_correct_html(content)


class TestIndexRender:

    def test_page_rendered_without_errors(self) -> None:

        language = "en"

        pages = []

        for i in range(10):
            article = library_make.article()
            page = library_make.page(article, body="bla-bla-text", language=language)
            pages.append(page)

        # TODO: make real meta_info
        #       maybe move code of constructing meta_info (from renderers.py) to separate function
        meta_info = MetaInfo(
            site_title="title",  # TODO: get real site title
            language=language,
            allowed_languages=[],
            title="seo title",
            seo_description="seo description",
            author="author",
            tags=[],  # TODO: add tags
            published_at=None,
            seo_image_url=None,
        )

        # TODO: test complex filter_state
        filter_state = UrlsTags(
            language=language,
            page=1,
            required_tags=[],
            excluded_tags=[],
        )

        info = Info(language=language, current_url=filter_state)

        index_info = IndexInfo(
            pages=pages,
            pages_found=len(pages),
            tags_count={},
        )

        request_context.set("language", language)
        request_context.set("url", filter_state)

        content = render(
            str(Template.index_page),
            {
                "info": info,
                "meta_info": meta_info,
                "index_info": index_info,
                "page_info": None,
            },
        )

        assert content is not None

        assert_correct_html(content)

    def test_more_link_rendered_only_if_part_of_content_is_hidden(self) -> None:
        language = "en"

        article_1 = library_make.article()
        page_1 = library_make.page(article_1, body="bla-bla-text", language=language)

        article_2 = library_make.article()
        page_2 = library_make.page(article_2, body="bla-bla-text <!-- more --> bla-bla-text", language=language)

        # TODO: make real meta_info
        #       maybe move code of constructing meta_info (from renderers.py) to separate function
        meta_info = MetaInfo(
            site_title="title",  # TODO: get real site title
            language=language,
            allowed_languages=[],
            title="seo title",
            seo_description="seo description",
            author="author",
            tags=[],  # TODO: add tags
            published_at=None,
            seo_image_url=None,
        )

        # TODO: test complex filter_state
        filter_state = UrlsTags(
            language=language,
            page=1,
            required_tags=[],
            excluded_tags=[],
        )

        info = Info(language=language, current_url=filter_state)

        pages = [page_1, page_2]

        index_info = IndexInfo(
            pages=pages,
            pages_found=len(pages),
            tags_count={},
        )

        request_context.set("language", language)
        request_context.set("url", filter_state)

        content = render(
            str(Template.index_page),
            {
                "info": info,
                "meta_info": meta_info,
                "index_info": index_info,
                "page_info": None,
            },
        )

        assert content is not None

        assert_correct_html(content)

        soup = BeautifulSoup(content, "html.parser")

        # rendered 2 pages, but only one of them has "more" link
        assert len(soup(class_="test-page-header-link")) == 2
        assert len(soup(class_="test-page-more-link")) == 1



import asyncio

from brigid.application.application import with_app
from brigid.cli.application import app
from brigid.core import logging
from brigid.validation import validators

logger = logging.get_module_logger()


async def run() -> None:
    async with with_app():
        # pages are loaded on the application initialization
        errors = validators.validate()

    if not errors:
        logger.info("everything_is_ok")
        return

    for error in errors:
        logger.error(
            "markdown_render_error",
            filepath=str(error.filepath),
            message=error.message,
        )

    logger.error("markdown_render_errors", errors_number=len(errors))


@app.command()
def validate() -> None:
    asyncio.run(run())



import asyncio
import pathlib
import sys

import typer

from brigid.application.application import with_app
from brigid.cli.application import app
from brigid.core import logging
from brigid.domain import request_context
from brigid.domain.urls import UrlsPlugin
from brigid.library.storage import storage
from brigid.plugins.utils import plugins

logger = logging.get_module_logger()


static_cli = typer.Typer()


async def run() -> None:
    async with with_app():
        files = []

        site = storage.get_site()

        with request_context.init():
            request_context.set("storage", storage)

            for plugin in plugins():
                url = UrlsPlugin(plugin.slug, language=site.default_language)

                for file_info in plugin.static_files():
                    full_path = pathlib.Path(file_info.sys_path).resolve()
                    files.append(f"{full_path}\t->\t{url.file_url(file_info.url_path)}")

        files.sort()

        sys.stdout.write("\n".join(files) + "\n")


@static_cli.command()
def list() -> None:
    asyncio.run(run())


app.add_typer(static_cli, name="static", help="Manage static files")



import pydantic_settings

from brigid.core.settings import BaseSettings


class Settings(BaseSettings):
    templates_reload: bool = False

    model_config = pydantic_settings.SettingsConfigDict(env_prefix="BRIGID_JINJA2_RENDER_")


settings = Settings()



import datetime
import enum
import pathlib
import re
from functools import cached_property
from typing import Literal

import pydantic

from brigid.core.entities import BaseEntity
from brigid.domain import urls
from brigid.domain.entities import Environment

MORE_RE = re.compile(r"<!--\s*more\s*-->", re.IGNORECASE)


class MenuItemType(enum.StrEnum):
    article = "article"
    feed = "feed"
    external = "external"
    blog = "blog"


class ArticleType(enum.StrEnum):
    post = "post"
    page = "page"


class OrderDirection(enum.StrEnum):
    asc = "asc"
    desc = "desc"


class MenuItemBase(BaseEntity):
    name: str
    language: str | None = None
    css_classes: str | None = None
    tooltip: str | None = None
    open_on_blank_page: bool = False

    model_config = pydantic.ConfigDict(frozen=False)

    def is_active(self, current_slug: str) -> bool:
        return False


class MenuItemArticle(MenuItemBase):
    type: Literal[MenuItemType.article]
    slug: str

    def is_active(self, current_slug: str) -> bool:
        return self.slug == current_slug

    @property
    def url(self) -> str:
        return urls.UrlsPost(language=self.language, slug=self.slug).url()


class MenuItemFeed(MenuItemBase):
    type: Literal[MenuItemType.feed]
    open_on_blank_page: bool = True

    @property
    def url(self) -> str:
        assert self.language is not None
        return urls.UrlsFeedsAtom(language=self.language).url()


class MenuItemExternal(MenuItemBase):
    type: Literal[MenuItemType.external]
    url: str
    open_on_blank_page: bool = True


class MenuItemBlog(MenuItemBase):
    type: Literal[MenuItemType.blog]

    def is_active(self, current_slug: str) -> bool:
        return current_slug is None

    @property
    def url(self) -> str:
        assert self.language is not None
        return urls.UrlsRoot(language=self.language).url()


MenuItem = MenuItemArticle | MenuItemFeed | MenuItemExternal | MenuItemBlog


class SimilarityConfig(BaseEntity):
    common_tag_score: int = 100
    referenced_from_score: int = 100
    referenced_to_score: int = 100
    bonus_per_reference: int = 10
    ignore_similar_tags: set[str] = pydantic.Field(default_factory=set)
    bonus_for_tags: dict[str, int] = pydantic.Field(default_factory=dict)


class License(BaseEntity):
    name: str
    url: str


class SiteLanguage(BaseEntity):
    title: str
    subtitle: str
    author: str
    license: License | None = None

    tags_translations: dict[str, str] = pydantic.Field(default_factory=dict)
    theme_translations: dict[str, str] = pydantic.Field(default_factory=dict)
    seo_translations: dict[str, str] = pydantic.Field(default_factory=dict)

    menu: list[MenuItem] = pydantic.Field(default_factory=list)


class Site(BaseEntity):
    prod_url: pydantic.HttpUrl
    local_url: pydantic.HttpUrl = pydantic.Field("http://0.0.0.0:8000")  # type: ignore
    default_language: str
    allowed_languages: set[str] = pydantic.Field(default_factory=set)
    posts_per_page: int = 5
    posts_in_feed: int = 10
    posts_in_latest: int = 20
    posts_in_similar: int = 6
    languages: dict[str, SiteLanguage] = pydantic.Field(default_factory=dict)
    favicon: str | None = None

    path: pathlib.Path

    content_repository: str | None = None

    show_brigid_link: bool = True

    similarity: SimilarityConfig = SimilarityConfig()

    model_config = pydantic.ConfigDict(frozen=False)

    @cached_property
    def url(self) -> str:
        from brigid.application.settings import settings

        if settings.environment == Environment.prod:
            return str(self.prod_url).rstrip("/")

        if settings.environment == Environment.local:
            return str(self.local_url).rstrip("/")

        raise NotImplementedError(f"Unknown environment: {settings.environment}")


class Article(BaseEntity):
    path: pathlib.Path

    slug: str

    type: ArticleType = ArticleType.post

    pages: dict[str, str] = pydantic.Field(default_factory=dict)

    tags: set[str] = pydantic.Field(default_factory=set)

    @cached_property
    def dir(self) -> pathlib.Path:
        return self.path.parent

    @cached_property
    def more_than_one_language(self) -> bool:
        return len(self.pages) > 1

    @cached_property
    def id(self) -> str:
        return f"article#{self.slug}"

    def first_language(self, *languages: str) -> str | None:
        for language in languages:
            if language in self.pages:
                return language

        return None


class Page(BaseEntity):
    article_id: str

    path: pathlib.Path

    # TODO: if in the future, than we hide the article?
    published_at: datetime.datetime

    language: str

    title: str

    # Google could show max from 155 to 160 characters from description in the search results
    # Facebook, Twitter could show more characters
    # But we'll stick to minimum from maximum
    seo_description: str = pydantic.Field(max_length=155)

    # could be None, but page should define it explicitly
    seo_image: str | None

    body: str

    tags: set[str] = pydantic.Field(default_factory=set)

    template: str | None = None

    series: str | None = None

    exclude: bool = False

    @cached_property
    def id(self) -> str:
        return f"#page#{self.slug}#{self.language}"

    @cached_property
    def slug(self) -> str:
        from brigid.library.storage import storage

        return storage.get_article(id=self.article_id).slug

    @cached_property
    def is_post(self) -> bool:
        from brigid.library.storage import storage

        return storage.get_article(id=self.article_id).type == ArticleType.post

    @cached_property
    def has_more(self) -> bool:
        return MORE_RE.search(self.body) is not None

    @cached_property
    def intro(self) -> str:
        if not self.has_more:
            return self.body

        return MORE_RE.split(self.body)[0]

    @cached_property
    def tags_in_translation_order(self) -> list[str]:
        from brigid.library.storage import storage

        translations = storage.get_site().languages[self.language].tags_translations

        tags = list(self.tags)
        tags.sort(key=lambda tag: translations[tag])

        return tags


class PageSimilarityScore(BaseEntity):
    page_id: str
    score: int

    explanations: list[str] = pydantic.Field(default_factory=list)

    model_config = pydantic.ConfigDict(frozen=False)

    def add_score(self, score: int, explanation: str) -> None:
        self.score += score
        self.explanations.append(f"{score}: {explanation}")


class Redirects(BaseEntity):
    permanent: dict[str, str] = pydantic.Field(default_factory=dict)


class Collection(BaseEntity):
    id: str
    path: pathlib.Path

    required_tags: set[str] = pydantic.Field(default_factory=set)
    excluded_tags: set[str] = pydantic.Field(default_factory=set)

    order: OrderDirection = OrderDirection.asc

    css_for_tags: dict[str, str] = pydantic.Field(default_factory=dict)

    def classes_for_tags(self, tags: set[str]) -> list[str]:
        classes = []

        for tag in tags:
            if tag in self.css_for_tags:
                classes.append(self.css_for_tags[tag])

        return classes


class PageSeriesInfo(BaseEntity):
    first_page: str
    prev_page: str | None
    next_page: str | None



import pathlib

import pydantic
import pydantic_settings

from brigid.core.settings import BaseSettings
from brigid.library.settings import settings as library_settings


def _default_static_extensions() -> dict[str, str]:
    return {
        "css": "text/css",
        "js": "application/javascript",
        "png": "image/png",
        "jpg": "image/jpeg",
        "jpeg": "image/jpeg",
        "gif": "image/gif",
        "svg": "image/svg+xml",
    }


class Settings(BaseSettings):
    static_base: pathlib.Path = pathlib.Path(__file__).parent / "static"
    static_redefined: pathlib.Path = library_settings.directory / "plugins" / "theme" / "static"
    static_extensions: dict[str, str] = pydantic.Field(default_factory=_default_static_extensions)

    templates_base: pathlib.Path = pathlib.Path(__file__).parent / "templates"
    templates_redefined: pathlib.Path = library_settings.directory / "plugins" / "theme" / "templates"

    model_config = pydantic_settings.SettingsConfigDict(env_prefix="BRIGID_PLUGINS_THEME_")


settings = Settings()



import pathlib

import pydantic_settings

from brigid.core.settings import BaseSettings
from brigid.library.settings import settings as library_settings


class Settings(BaseSettings):
    templates: pathlib.Path = library_settings.directory / "plugins" / "include" / "templates"

    model_config = pydantic_settings.SettingsConfigDict(env_prefix="BRIGID_PLUGINS_INCLUDE_")


settings = Settings()



from urllib.parse import urlparse

from markdown.inlinepatterns import LINK_RE as EXTERNAL_LINK_RE
from markdown.inlinepatterns import LinkInlineProcessor

from brigid.library.storage import storage
from brigid.markdown_render.context import render_context


class ExternalLinkInlineProcessor(LinkInlineProcessor):

    # TODO: return errors instead of result
    def handleMatch(self, m, data):  # noqa # pylint: disable=all
        result = super().handleMatch(m, data)

        if result[0] is None:
            return result

        href = result[0].get("href")

        if href is None:
            return result

        parsed_url = urlparse(href)

        site = storage.get_site()

        context = render_context.get()

        parsed_site = urlparse(site.url)

        # validate external links

        # TODO: compare taking into account the default ports, aka example.com:90 = example.com
        # TODO: is this required, link could be to the same domain but in different project
        if parsed_site.netloc == parsed_url.netloc:
            context.add_error(
                failed_text=data, message="Use internal links syntax to specify links to the same domain"
            )
            return result

        if parsed_url.scheme == "":
            context.add_error(failed_text=data, message="Specify schema/protocol for external links")
            return result

        # all external links must be with target="_blank"
        result[0].set("target", "_blank")

        return result


__all__ = ["ExternalLinkInlineProcessor", "EXTERNAL_LINK_RE"]



import jinja2

from brigid.plugins.include.settings import settings
from brigid.plugins.plugin import Plugin


class CorePlugin(Plugin):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def templates_loader(self) -> jinja2.BaseLoader | None:
        return jinja2.FileSystemLoader(settings.templates, followlinks=True)


plugin = CorePlugin(slug="include")



import pydantic
import pydantic_settings

from brigid.core.settings import BaseSettings
from brigid.domain.entities import Environment


class Sentry(pydantic.BaseModel):
    enabled: bool = False
    dsn: str = ""
    sample_rate: float = 1.0


_local_origins = ("*",)


class Settings(BaseSettings):
    environment: Environment = Environment.local

    sentry: Sentry = Sentry()

    origins: tuple[str, ...] = _local_origins

    @pydantic.model_validator(mode="after")
    def origin_must_be_redefined_in_prod(self) -> "Settings":
        if self.environment == Environment.prod and self.origins == _local_origins:
            raise ValueError("Origins must be redefined in prod")

        return self

    model_config = pydantic_settings.SettingsConfigDict(env_prefix="BRIGID_")


settings = Settings()



from fastapi import HTTPException


class PageNotFound(HTTPException):
    def __init__(self, **kwargs) -> None:
        super().__init__(status_code=404, **kwargs)


class FileNotFound(HTTPException):
    def __init__(self, **kwargs) -> None:
        super().__init__(status_code=404, **kwargs)



import jinja2

from brigid.plugins.photoswipe.settings import settings
from brigid.plugins.plugin import Plugin


class CorePlugin(Plugin):
    def __init__(self, settings, **kwargs):
        self.settings = settings
        super().__init__(**kwargs)

    def templates_loader(self) -> jinja2.BaseLoader:
        return jinja2.FileSystemLoader(settings.templates, followlinks=True)


plugin = CorePlugin(slug="photoswipe", settings=settings)



import datetime
import enum

import pydantic

from brigid.core.entities import BaseEntity
from brigid.domain.urls import UrlsBase
from brigid.library.entities import Article, Page, PageSimilarityScore


# TODO: template names should have prefixes
class Template(enum.StrEnum):
    article_page = "theme/article.html.j2"
    index_page = "theme/blog_index.html.j2"


class Info(BaseEntity):
    language: str
    current_url: UrlsBase

    model_config = pydantic.ConfigDict(
        arbitrary_types_allowed=True,
    )


class MetaInfo(BaseEntity):
    site_title: str

    language: str
    allowed_languages: list[str]

    title: str
    seo_description: str
    author: str
    tags: list[str]
    published_at: datetime.datetime | None

    seo_image_url: str | None


class IndexInfo(BaseEntity):
    pages: list[Page]
    pages_found: int
    tags_count: dict[str, int]


class PageInfo(BaseEntity):
    article: Article
    page: Page
    similar_pages: list[PageSimilarityScore]



import pathlib

import pydantic_settings

from brigid.core.settings import BaseSettings


class Settings(BaseSettings):
    templates: pathlib.Path = pathlib.Path(__file__).parent / "templates"

    stylesheet: str = "https://cdnjs.cloudflare.com/ajax/libs/photoswipe/5.4.2/photoswipe.min.css"
    lightbox: str = "https://cdnjs.cloudflare.com/ajax/libs/photoswipe/5.4.2/photoswipe-lightbox.esm.min.js"
    pswp: str = "https://cdnjs.cloudflare.com/ajax/libs/photoswipe/5.4.2/photoswipe.esm.min.js"

    model_config = pydantic_settings.SettingsConfigDict(env_prefix="BRIGID_PLUGINS_PHOTOSWIPE_")


settings = Settings()



from brigid.domain import request_context
from brigid.library.storage import storage
from brigid.validation.entities import Error
from brigid.validation.global_validators import required_article
from brigid.validation.page_validators import page_has_correct_series_tags, page_has_correct_tags, page_is_rendered

page_validators = [
    page_is_rendered,
    page_has_correct_tags,
    page_has_correct_series_tags,
]

global_validators = [required_article("404"), required_article("500")]


def validate() -> list[Error]:
    errors = []

    for global_validator in global_validators:
        with request_context.init():
            request_context.set("storage", storage)
            errors.extend(global_validator())

    for page in storage.all_entities():
        for page_validator in page_validators:
            with request_context.init():
                request_context.set("storage", storage)
                errors.extend(page_validator(page))

    return errors



from brigid.library.entities import Page, PageSeriesInfo
from brigid.library.storage import storage


def get_page_series_info(original_page: Page) -> PageSeriesInfo:

    if original_page.series is None:
        raise NotImplementedError("Only pages with series are supported")

    series_pages: list[str] = []

    for page in storage.get_posts(language=original_page.language):
        if original_page.series != page.series:
            continue

        series_pages.append(page.id)

    series_pages.sort(key=lambda p_id: storage.get_page(p_id).published_at)

    first_page = series_pages[0]
    prev_page = None
    next_page = None

    original_page_index = series_pages.index(original_page.id)

    if original_page_index > 0:
        prev_page = series_pages[original_page_index - 1]

    if original_page_index < len(series_pages) - 1:
        next_page = series_pages[original_page_index + 1]

    return PageSeriesInfo(
        first_page=first_page,
        prev_page=prev_page,
        next_page=next_page,
    )



from brigid.library.entities import Page
from brigid.library.storage import storage
from brigid.markdown_render.markdown_render import render_page
from brigid.validation.entities import Error


def page_is_rendered(page: Page) -> list[Error]:
    context = render_page(page=page)

    errors = []

    for error in context.errors:
        errors.append(Error(filepath=page.path, message=f'Render error "{error.message}" in "{error.failed_text}"'))

    return errors


def page_has_correct_tags(page: Page) -> list[Error]:
    errors = []

    allowed_tags = set(storage.get_site().languages[page.language].tags_translations.keys())

    for tag in page.tags:
        if tag not in allowed_tags:
            errors.append(
                Error(filepath=page.path, message=f"Tag {tag} is not registered for site language {page.language}")
            )

    return errors


def page_has_correct_series_tags(page: Page) -> list[Error]:
    if page.series is None:
        return []

    errors = []

    allowed_series_tags = set(storage.get_site().languages[page.language].tags_translations.keys())

    if page.series not in page.tags:
        errors.append(Error(filepath=page.path, message=f"Series tag {page.series} should be also in page tags"))

    if page.series not in allowed_series_tags:
        errors.append(
            Error(
                filepath=page.path,
                message=f"Series tag {page.series} is not registered for site language {page.language}",
            )
        )

    return errors



import pytest

from brigid.domain.text import capitalize_first


class TestCapitalizeFirst:

    @pytest.mark.parametrize(
        "text_input,text_expected",
        [
            ["", ""],
            ["tst test", "Tst test"],
            ["?sadasd", "?sadasd"],
            ["ABBR some text ABBR some text ABBR", "ABBR some text ABBR some text ABBR"],
        ],
    )
    def test(self, text_input: str, text_expected: str) -> None:
        assert capitalize_first(text_input) == text_expected



import pydantic_settings

from brigid.core.settings import BaseSettings


class Settings(BaseSettings):
    plugins: list[str] = [
        "brigid.plugins.core.plugin:plugin",
        "brigid.plugins.i18n.plugin:plugin",
        "brigid.plugins.seo.plugin:plugin",
        "brigid.plugins.include.plugin:plugin",
        "brigid.plugins.theme.plugin:plugin",
        "brigid.plugins.photoswipe.plugin:plugin",
    ]

    model_config = pydantic_settings.SettingsConfigDict(env_prefix="BRIGID_PLUGINS_")


settings = Settings()



import asyncio
import os
from typing import AsyncGenerator, Generator

import fastapi
import pytest
import pytest_asyncio
from fastapi.testclient import TestClient

from brigid.application import application
from brigid.domain import request_context
from brigid.library.storage import storage


@pytest.fixture(scope="session", autouse=True)
def mark_tests_running():
    os.environ["BRIGID_TESTS_RUNNING"] = "True"


@pytest.fixture(scope="session", autouse=True)
def event_loop() -> Generator[asyncio.AbstractEventLoop, asyncio.AbstractEventLoop, None]:
    loop = asyncio.get_event_loop_policy().new_event_loop()
    yield loop
    loop.close()


@pytest.fixture(autouse=True)
def reset_request_context():
    with request_context.init():
        request_context.set("storage", storage)
        yield


@pytest_asyncio.fixture(scope="session", autouse=True)
async def app() -> AsyncGenerator[fastapi.FastAPI, None]:
    async with application.with_app() as app:
        yield app


@pytest.fixture
def client(app: fastapi.FastAPI) -> Generator[TestClient, None, None]:
    yield TestClient(app, raise_server_exceptions=True, follow_redirects=False)



from sentry_sdk import init as initialize_sentry
from sentry_sdk.integrations.fastapi import FastApiIntegration
from sentry_sdk.integrations.logging import LoggingIntegration
from sentry_sdk.integrations.starlette import StarletteIntegration

import brigid


def initialize(dsn: str, sample_rate: float, environment: str) -> None:
    initialize_sentry(
        dsn=dsn,
        sample_rate=sample_rate,
        traces_sample_rate=None,
        max_breadcrumbs=0,
        # Without this config Sentry miss important frames from stacktraces
        add_full_stack=True,
        attach_stacktrace=True,
        environment=environment,
        # set the correct project root directory
        project_root=brigid.__path__[0],
        # disable ALL automatically enabled integrations, because they periodically cause issues
        auto_enabling_integrations=False,
        integrations=[
            StarletteIntegration(transaction_style="endpoint"),
            FastApiIntegration(transaction_style="endpoint"),
        ],
        disabled_integrations=[
            # disable default logging integration to use specialized structlog-sentry processor
            LoggingIntegration(event_level=None, level=None),
        ],
    )



import pathlib

from brigid.core.entities import BaseEntity


class FileInfo(BaseEntity):
    sys_path: pathlib.Path
    url_path: str
    media_type: str

    @property
    def is_stylesheet(self) -> bool:
        return self.media_type == "text/css"

    @property
    def is_javascript(self) -> bool:
        return self.media_type == "application/javascript"
